 /****************************************************************************
 **
 ** Copyright (C) 2005-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the example classes of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 2.0 as published by the Free Software Foundation
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
 ** http://www.trolltech.com/products/qt/opensource.html
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://www.trolltech.com/products/qt/licensing.html or contact the
 ** sales department at sales@trolltech.com.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/

#include <QtGui>
#include <QRegExp>
#include "highlighter.h"

Highlighter::Highlighter(QTextDocument *parent) : QSyntaxHighlighter(parent) {
	HighlightingRule rule;
	
	numberFormat.setForeground(QColor(215, 70, 0));
	rule.pattern = QRegExp("\\b[0-9]+\\b");
	rule.format = numberFormat;
	highlightingRules.append(rule);

	keywordFormat.setForeground(QColor(0, 0, 202));
	rule.pattern = QRegExp("\\b(Abs|ACos|AddText|After|And|AnimationHeight|AnimationPlaying|AnimationWidth|As|Asc|ASin|ATan|Before|Bin|BLUE|Box|BoxOverlap|Byte|CallDLL|CameraAngle|CameraFollow|CameraPick|CameraX|CameraY|Case|CenterText|ChDir|Chr|Circle|ClearArray|ClearCollisions|ClearKeys|ClearMouse|ClearObjects|ClearText|CloneCameraOrientation|CloneCameraPosition|CloneImage|CloneObject|CloneObjectOrientation|CloneObjectPosition|CloseFile|CloseInput|Cls|ClsColor|CollisionAngle|CollisionX|CollisionX|Color|CommandLine|Const|ConvertToInteger|ConvertToType|CopyBox|CopyFile|Cos|CountCollisions|CountWords|Crc32|CurrentDir|CurveAngle|CurveValue|Data|Date|Decrypt|Default|DefaultMask|DefaultVisible|Delete|DeleteFile|DeleteFont|DeleteImage|DeleteMEMBlock|DeleteObject|DeleteSound|Dim|Distance|Distance2|Dot|DownKey|DrawAnimation|DrawGame|DrawGhostImage|DrawImage|DrawImageBox|DrawScreen|DrawToImage|DrawToScreen|DrawToWorld|Each|EditMap|Ellipse|Else|ElseIf|Encrypt|End|EndFunction|EndIf|EndSelect|EndType|EndSearch|EOF|Errors|EscapeKey|Execute|Exit|False|Field|FileExists|FileOffset|FileSize|FindFile|First|Flip|Float|For|Forever|FPS|FrameLimit|Function|GetAngle|GetAngle2|GetCollision|GetEXEName|GetKey|GetMap|GetMap2|GetMouse|GetPixel|GetPixel2|getRGB|GetWord|GFXModeExists|GhostObject|Global|Gosub|Goto|GotoSavedLocation|GREEN|Hex|HotSpot|If|Image|ImageHeight|ImagesCollide|ImagesOverlap|ImageWidth|Include|InitObjectList|Input|Insert|InStr|Int|Integer|IsDirectory|KeyDown|KeyHit|KeyUp|Last|Left|LeftKey|Len|Line|LoadAnimImage|LoadAnimObject|LoadFont|LoadImage|LoadMap|LoadObject|LoadProgram|LoadSound|Locate|Lock|Log|Log10|LoopObject|Lower|LSet|MakeDir|MakeEmitter|MakeError|MakeImage|MakeMap|MakeMEMBlock|MakeObject|MakeObjectFloor|MapHeight|MapWidth|MaskImage|MaskObject|Max|MEMBlockSize|MemCopy|Min|Mid|MirrorObject|Mod|MouseDown|MouseHit|MouseMoveX|MouseMoveY|MouseMoveZ|MouseUp|MouseWX|MouseWY|MouseX|MouseY|MouseX|MoveCamera|MoveObject|New|Next|NextObject|Not|NULL|ObjectAngle|ObjectFloat|ObjectFrame|ObjectInteger|ObjectLife|ObjectOrder|ObjectPick|ObjectPickable|ObjectPlaying|ObjectRange|ObjectSight|ObjectSizeX|ObjectSizeY|ObjectsOverlap|ObjectString|ObjectX|ObjectY|OFF|ON|OpenToEdit|OpenToRead|OpenToWrite|Or|PaintObject|ParticleAnimation|ParticleEmission|ParticleMovement|PeekByte|PeekFloat|PeekInt|PeekShort|PI|PickColor|PickedAngle|PickedObject|PickedX|PickedY|PickImageColor|PickImageColor2|PixelPick|PlayAnimation|PlayObject|PlaySound|PointCamera|PointObject|PokeByte|PokeFloat|PokeInt|PokeShort|PositionCamera|PositionMouse|PositionObject|Print|PutPixel|PutPixel2|Rand|Randomize|Read|ReadByte|ReadFloat|ReadInt|ReadLine|ReadShort|ReadString|RED|ReDim|Repeat|Replace|ResetObjectCollision|ResizeImage|ResizeMEMBlock|Restore|Return|Right|RightKey|Rnd|RotateCamera|RotateImage|RotateObject|RoundDown|RoundUp|RSet|SAFEEXIT|Sar|SaveImage|SaveProgram|SCREEN|ScreenDepth|ScreenGamma|ScreenHeight|ScreenPositionObject|ScreenShot|ScreenWidth|SeekFile|Select|SetFont|SetMap|SetSound|SetTile|SetupCollision|SetWindow|Shl|Short|ShowMouse|ShowObject|Shr|Sin|Smooth2D|SoundPlaying|Sqrt|StartSearch|Step|StopAnimation|StopObject|StopSound|Str|String|StrInsert|StrMove|StrRemove|Tan|Text|TextHeight|TextWidth|Then|Time|Timer|To|TranslateCamera|TranslateObject|Trim|True|TurnCamera|TurnObject|Type|Unlock|Until|UpdateGame|UpKey|Upper|VerticalText|Wait|WaitKey|WaitMouse|Wend|While|WrapAngle|Write|WriteByte|WriteFloat|WriteInt|WriteLine|WriteShort|WriteString|Xor)\\b");
	rule.format = keywordFormat;
	highlightingRules.append(rule);
	
	operatorFormat.setForeground(QColor(12, 87, 133));
	rule.pattern = QRegExp("[+/*^=<>()-]");
	rule.format = operatorFormat;
	highlightingRules.append(rule);
	
	quotationFormat.setForeground(QColor(0, 176, 0));
	//rule.pattern = QRegExp("\".*\"");
	//rule.format = quotationFormat;
	//highlightingRules.append(rule);

	singleLineCommentFormat.setForeground(QColor(166, 166, 166));
	//rule.pattern = QRegExp("(//|')[^\n]*");
	//rule.format = singleLineCommentFormat;
	//highlightingRules.append(rule);

	multiLineCommentFormat.setForeground(QColor(166, 166, 166));

	commentStartExpression = QRegExp("\\bremstart\\b");
	commentEndExpression = QRegExp("\\bremend\\b");

	commentStartExpression.setCaseSensitivity(Qt::CaseInsensitive);
	commentEndExpression.setCaseSensitivity(Qt::CaseInsensitive);
}

void Highlighter::highlightBlock(const QString &text) {
	foreach (HighlightingRule rule, highlightingRules) {
		QRegExp expression(rule.pattern);
		int index = text.indexOf(expression);
		while (index >= 0) {
			int length = expression.matchedLength();
			setFormat(index, length, rule.format);
			index = text.indexOf(expression, index + length);
		}
	}
	
	QRegExp expression("\"");
	int index = text.indexOf(expression);
	while (index >= 0) {
		int endIndex = text.indexOf(expression, index + 1);
		int length;
		if (endIndex == -1) {
			length = text.length() - index;
		}
		else {
			length = endIndex - index + expression.matchedLength();
		}
		setFormat(index, length, quotationFormat);
		index = text.indexOf(expression, index + length);
	}

	expression = QRegExp("//");
	index = text.indexOf(expression);
	if (index >= 0) {
		setFormat(index, text.length() - index, singleLineCommentFormat);
	}

	setCurrentBlockState(0);

	int startIndex = 0;
	if (previousBlockState() != 1)
		startIndex = text.indexOf(commentStartExpression);

	while (startIndex >= 0) {
		int endIndex = text.indexOf(commentEndExpression, startIndex);
		int commentLength;
		if (endIndex == -1) {
			setCurrentBlockState(1);
			commentLength = text.length() - startIndex;
		} else {
			commentLength = endIndex - startIndex + commentEndExpression.matchedLength();
		}
		setFormat(startIndex, commentLength, multiLineCommentFormat);
		startIndex = text.indexOf(commentStartExpression, startIndex + commentLength);
	}
}
