#include "codetab.h"
#include "mainwindow.h"
#include "highlighter.h"
#include "ui_mainwindow.h"
#include <QCloseEvent>
#include <QFileDialog>
#include <QByteArray>
#include <QMessageBox>
#include <QFile>
#include <QTextStream>

MainWindow::MainWindow(QWidget *parent, Qt::WFlags flags) : QMainWindow(parent, flags) {
	setupUi(this);

	CodeTab *codeTab = static_cast<CodeTab *>(tabWidget->currentWidget());
	highlighter = new Highlighter(codeTab->getCodeEditor()->document());
	codeTab->setTabWidget(tabWidget);
	tabWidget->setTabText(0, "<New>");

	actionNew->setShortcut(tr("Ctrl+N"));
	actionOpen->setShortcut(tr("Ctrl+O"));
	actionSave->setShortcut(tr("Ctrl+S"));
	actionSave_As->setShortcut(tr("Ctrl+F12"));
	actionSave_All->setShortcut(tr("Ctrl+Shift+S"));
	actionClose->setShortcut(tr("Ctrl+W"));
	actionUndo->setShortcut(tr("Ctrl+Z"));
	actionRedo->setShortcut(tr("Ctrl+Y"));
	actionCut->setShortcut(tr("Ctrl+X"));
	actionCopy->setShortcut(tr("Ctrl+C"));
	actionPaste->setShortcut(tr("Ctrl+V"));
	actionSelect_all->setShortcut(tr("Ctrl+A"));
	actionCheck_Syntax->setShortcut(tr("F4"));
	actionCompile_and_RUN->setShortcut(tr("F5"));
	actionBuild_EXE->setShortcut(tr("F7"));

	menuHelp->addAction(dockManual->toggleViewAction());	

	LoadSettings();
}

MainWindow::~MainWindow() {
	SaveSettings();
}

void MainWindow::LoadSettings() {
	mainSettings.beginGroup(QLatin1String("mainwindow"));
    QByteArray geom = mainSettings.value(QLatin1String("geometry")).toByteArray();
    if (!geom.isEmpty()) {
        restoreGeometry(geom);
	}
    else {
        resize(800, 600);
	}
	restoreState(mainSettings.value(QLatin1String("state"),QByteArray()).toByteArray());
	mainSettings.endGroup();
}

void MainWindow::SaveSettings() {
	mainSettings.beginGroup(QLatin1String("mainwindow"));
    mainSettings.setValue(QLatin1String("geometry"), saveGeometry());
    mainSettings.setValue(QLatin1String("state"), saveState());
    mainSettings.endGroup();
}

void MainWindow::OpenFile(QString path) {
	if (path.isEmpty()) {
		return;
	}

	CodeTab *codeTab = new CodeTab(tabWidget, path);
	tabWidget->setCurrentIndex(tabWidget->addTab(codeTab, codeTab->getFileName()));
	Highlighter *ligter = new Highlighter(codeTab->getCodeEditor()->document());
}

void MainWindow::closeEvent(QCloseEvent *event) {
	event->accept();
}

void MainWindow::handleHomeButton() {
	webView->load(QString("Help/index.html"));
}

void MainWindow::handleActionNew() {
	CodeTab *codeTab = new CodeTab(tabWidget);
	tabWidget->setCurrentIndex(tabWidget->addTab(codeTab, "<New>"));
	Highlighter *ligter = new Highlighter(codeTab->getCodeEditor()->document());
}

void MainWindow::handleActionOpen() {
	OpenFile(QFileDialog::getOpenFileName(this, tr("Open"), QString(), tr("CoolBasic Source (*.cb);;Text files (*.txt);;All files (*)")));
}

void MainWindow::handleActionSave() {
	SaveFile(static_cast<CodeTab *>(tabWidget->currentWidget()));
}

void MainWindow::handleActionSave_As() {
	SaveFileAs(static_cast<CodeTab *>(tabWidget->currentWidget()));
}

void MainWindow::handleActionSave_All() {
	for (int i = 0; i < tabWidget->count(); i++) {
		SaveFile(static_cast<CodeTab *>(tabWidget->widget(i)));
	}
}

void MainWindow::handleActionClose() {
	CloseTab(static_cast<CodeTab *>(tabWidget->currentWidget()));
}

void MainWindow::handleActionClose_All() {
	while (tabWidget->count() > 0) {
		if (!CloseTab(static_cast<CodeTab *>(tabWidget->widget(tabWidget->count() - 1)))) {
			break;
		}
	}
}

void MainWindow::handleActionUndo() {
	static_cast<CodeTab *>(tabWidget->currentWidget())->getCodeEditor()->undo();
}

void MainWindow::handleActionRedo() {
	static_cast<CodeTab *>(tabWidget->currentWidget())->getCodeEditor()->redo();
}

void MainWindow::handleActionCut() {
	static_cast<CodeTab *>(tabWidget->currentWidget())->getCodeEditor()->cut();
}

void MainWindow::handleActionCopy() {
	static_cast<CodeTab *>(tabWidget->currentWidget())->getCodeEditor()->copy();
}

void MainWindow::handleActionPaste() {
	static_cast<CodeTab *>(tabWidget->currentWidget())->getCodeEditor()->paste();
}

void MainWindow::handleActionSelect_All() {
	static_cast<CodeTab *>(tabWidget->currentWidget())->getCodeEditor()->selectAll();
}

void MainWindow::handleActionCheck_Syntax() {
	CodeTab *codeTab = static_cast<CodeTab *>(tabWidget->currentWidget());
	QFile file("IDE/Compiler");
	if (file.open(QFile::WriteOnly | QFile::Text)) {
		QTextStream out(&file);
		out << "type=0" << "\n" << "sourcedir=" << codeTab->getDirectory() << "\n" << "buildto=" << "\n" << "force=0" << "\n";
		file.close();
	}
	
	QFile file2("IDE/Editor.out");
	if (file2.open(QFile::WriteOnly | QFile::Text)) {

		QString data = codeTab->getCodeEditor()->toPlainText();
		data.replace("\n", "\r\n");
		file2.write(data.toAscii(), data.length());
		file2.close();	
	}

	system("wine ./IDE/CBCompiler.exe");
}

void MainWindow::handleActionCompile_and_RUN() {
	CodeTab *codeTab = static_cast<CodeTab *>(tabWidget->currentWidget());
	QFile file("IDE/Compiler");
	if (file.open(QFile::WriteOnly | QFile::Text)) {
		QTextStream out(&file);
		out << "type=1" << "\n" << "sourcedir=" << codeTab->getDirectory() << "\n" << "buildto=" << QDir::currentPath() << "/IDE/cbRun.exe" << "\n" << "force=0" << "\n";
		file.close();
	}
	
	QFile file2("IDE/Editor.out");
	if (file2.open(QFile::WriteOnly | QFile::Text)) {

		QString data = codeTab->getCodeEditor()->toPlainText();
		data.replace("\n", "\r\n");
		file2.write(data.toAscii(), data.length());
		file2.close();	
	}

	system("wine ./IDE/CBCompiler.exe");
	system("wine ./IDE/cbRun.exe &");
}

void MainWindow::handleActionBuild_EXE() {
	QFileDialog dialog(this);
	dialog.setFileMode(QFileDialog::AnyFile);
	dialog.setNameFilter("Windows shitty .exe files :( (*.exe)");
	dialog.setAcceptMode(QFileDialog::AcceptSave);
	dialog.setDefaultSuffix(".exe");

	if (!dialog.exec()) {
		return;
	}
	QString filename = dialog.selectedFiles().at(0);

	CodeTab *codeTab = static_cast<CodeTab *>(tabWidget->currentWidget());
	QFile file("IDE/Compiler");
	if (file.open(QFile::WriteOnly | QFile::Text)) {
		QTextStream out(&file);
		out << "type=1" << "\n" << "sourcedir=" << codeTab->getDirectory() << "\n" << "buildto=" << filename << "\n" << "force=0" << "\n";
		file.close();
	}
	
	QFile file2("IDE/Editor.out");
	if (file2.open(QFile::WriteOnly | QFile::Text)) {

		QString data = codeTab->getCodeEditor()->toPlainText();
		data.replace("\n", "\r\n");
		file2.write(data.toAscii(), data.length());
		file2.close();	
	}

	system("wine ./IDE/CBCompiler.exe");
}

bool MainWindow::CloseTab(CodeTab *codeTab) {
	if (ConfirmSave(codeTab)) {
		tabWidget->removeTab(tabWidget->currentIndex());
		delete codeTab;

		if (tabWidget->count() == 0) {
			handleActionNew();
		}
		return true;
	}
	return false;
}

bool MainWindow::ConfirmSave(CodeTab *codeTab) {
	if (!codeTab || codeTab->isSaved()) {
		return true;
	}

	int ret = QMessageBox::warning(this, tr("Close"), tr("There are unsaved changes. Do you want to save now?"), QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
	switch (ret) {
		case QMessageBox::Save: return SaveFile(codeTab);
		case QMessageBox::Discard: return true;
		default:
			return false;
	}
}

bool MainWindow::SaveFile(CodeTab *codeTab) {
	if (codeTab->getFileName() == QString()) {
		return SaveFileAs(codeTab);
	}

	codeTab->save();

	return false;
}

bool MainWindow::SaveFileAs(CodeTab *codeTab) {
	QFileDialog dialog(this);
	dialog.setFileMode(QFileDialog::AnyFile);
	dialog.setNameFilter("CoolBasic Source (*.cb);;Text files (*.txt);;All files (*)");
	dialog.setAcceptMode(QFileDialog::AcceptSave);
	dialog.setDefaultSuffix(".cb");

	if (dialog.exec()) {
		QStringList files = dialog.selectedFiles();

		codeTab->setFilePath(files.at(0));
		codeTab->save();

		return true;
	}
	return false;
}
